/********************************************************

Esempio pilotaggio CENT4UR con Arduino
(C)2010 Giovanni Bernardo

http://www.settorezero.com/wordpress/servizi/cent4ur/

Collegamenti:
Pin CENT4UR  / Pin Arduino
GND            GND
PWA            9
PWB            10
SENA           A1
SENB           A2

Collegare il polo centrale di un trimmer/potenziometro 
lineare da 10K su A0 di Arduino. I pin laterali del
potenziometro vanno tra 5V 3 GND.

Arduino viene impostato per avere un segnale PWM a 
31,25KHz sui pin 9 e 10. Ruotando il potenziometro 
si varia il duty cycle. CENT4UR viene pilotato in
modalità LAP. Aprendo il terminale seriale a 9600bps
è possibile leggere il valore di duty cycle e il valore
di assorbimento dei due motori, espresso in mA

********************************************************/

#define MotorA  9
#define MotorB  10

void setup() 
  {
  Serial.begin(9600);
  // PWM sui pin 9 e 10 a 31,2KHz
  setPwmFrequency(MotorA, 1);
  setPwmFrequency(MotorB, 1);    
  }

void loop() 
  {
  // lettura potenziometro su A0
  unsigned int PotValue = analogRead(A0);
  unsigned char PWMValue; // valore di duty cycle
  // scalo il valore della lettura analogica, 10bit
  // al valore da assegnare al duty cycle, 8bit
  PWMValue = map(PotValue, 0, 1023, 0, 255);  
  // imposto il duty cycle sulle due uscite
  analogWrite(MotorA, PWMValue);
  analogWrite(MotorB, PWMValue);
  // lettura current sensing cent4ur
  unsigned int CurrentA = analogRead(A1);
  unsigned int CurrentB = analogRead(A2);
  // stampo i valori su seriale
  Serial.print("Duty cycle: ");
  Serial.print(PWMValue, DEC);
  Serial.print(" Assorbimento motore A: ");
  Serial.print(CurrentA*10, DEC);
  Serial.print("mA, motore B: ");
  Serial.print(CurrentB*10, DEC);
  Serial.println("mA");
  }


void setPwmFrequency(int pin, int divisor) 
  {
  byte mode;
  if(pin == 5 || pin == 6 || pin == 9 || pin == 10) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 64: mode = 0x03; break;
      case 256: mode = 0x04; break;
      case 1024: mode = 0x05; break;
      default: return;
    }
    if(pin == 5 || pin == 6) {
      TCCR0B = TCCR0B & 0b11111000 | mode;
    } else {
      TCCR1B = TCCR1B & 0b11111000 | mode;
    }
  } else if(pin == 3 || pin == 11) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 32: mode = 0x03; break;
      case 64: mode = 0x04; break;
      case 128: mode = 0x05; break;
      case 256: mode = 0x06; break;
      case 1024: mode = 0x7; break;
      default: return;
    }
    TCCR2B = TCCR2B & 0b11111000 | mode;
  }
}
